from ._weather_query import weather_query as today_weather
from ._thesis_info import thesis_info as crawl_thesis_info
from ._ytb_channel_info import YTBInfoBot
from ._academic import PaperWithCodeQuery, ArxivQuery, IEEEQuery
from ._air_quality import AirQualityQuery
from ._exchange_rate_cn import exchange_rate_cn
from ._realpython_news import RealPythonNews
from ._abstracts_arxiv import query_abstracts_arxiv
from . import openpy
from . import asyncpy